# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# U8MG - an utf8 migration tool
#
# Copyright (C) 2005 Tollef Fog Heen
#
# Wizards taken from flumotion,
# Copyright (C) 2004,2005 Fluendo, S.L. (www.fluendo.com). All rights reserved.

# This file may be distributed and/or modified under the terms of
# the GNU General Public License version 2 as published by
# the Free Software Foundation.
# This file is distributed without any warranty; without even the implied
# warranty of merchantability or fitness for a particular purpose.
# See "LICENSE.GPL" in the source distribution for more information.

import os
import sets

import gobject
import gtk
import gtk.gdk
import gtk.glade
import configure

def escape(text):
    return text.replace('&', '&amp;')

class Stack(list):
    push = list.append
    def peek(self):
        return self[-1]
    

class WizardStep(object):
    step_name = None # Subclass sets this
    glade_file = None # Subclass sets this
    icon = 'placeholder.png'
    widget_prefixes = { gtk.ComboBox    : 'combobox',
                        gtk.CheckButton : 'checkbutton',
                        gtk.Entry       : 'entry',
                        gtk.SpinButton  : 'spinbutton',
                        gtk.RadioButton : 'radiobutton' }


    def __init__(self, wizard):
        self.wizard = wizard
        self.widget = None
        self.visited = False
        
        self.load_glade()

    def __repr__(self):
        return '<WizardStep object %s>' % self.step_name
    
    def load_glade(self):
        glade_filename = os.path.join(configure.gladedir, self.glade_file)
        
        self.wtree = gtk.glade.XML(glade_filename)
        
        windows = []
        self.widgets = self.wtree.get_widget_prefix('')
        for widget in self.widgets:
            # So we can access the step from inside the widget
            widget.set_data('wizard-step', self)

            if isinstance(widget, gtk.Window):
                if widget.get_property('visible'):
                    raise AssertionError('window for %r is visible' % self)
                widget.hide()
                windows.append(widget)
                continue
            
            name = widget.get_name()
            if hasattr(self, name):
                raise AssertionError("There is already an attribute called %s in %r" % (name, self))
            
            setattr(self, name, widget)

        if len(windows) != 1:
            raise AssertionError("only one window per glade file allowed, got %r in %r" % (
                windows, self))

        self.window = windows[0]
        child = self.window.get_children()[0]
        self.window.remove(child)
        self.widget = child

        # And at last, connect signals.
        self.wtree.signal_autoconnect(self)
        
    def get_component_properties(self):
        return self.wizard.get_step_state(self)
    
    def get_main_widget(self):
        return self.widget

    def get_state(self):
        state_dict = {}
        for widget in self.widgets:
            name = widget.get_name()
            prefix = self.widget_prefixes.get(widget.__class__, None)
            if not prefix:
                continue
            try:
                key = name.split('_', 1)[1]
            except IndexError:
                continue
            
            state_dict[key] = widget

        return state_dict

    def get_name(self):
        return self.step_name
    
    def get_sidebar_name(self):
        return getattr(self, 'sidebar_name', self.step_name)

    def get_section(self):
        return getattr(self, 'section', '')
       
    def get_next(self):
        """
        @returns name of next step
        @rtype   string

        This is called when the user presses next in the wizard,
        
        A subclass must implement this"""
        
        raise NotImplementedError

    def activated(self):
        """Called just before the step is shown, so the step can
        do some logic, eg setup the default state

        This can be implemented in a subclass"""
        
    def deactivated(self):
        """Called after the user pressed next

        This can be implemented in a subclass"""

    def setup(self):
        """This is called after the step is constructed, to be able to
        do some initalization time logic in the steps.

        This can be implemented in a subclass."""


    def before_show(self):
        """This is called just before we show the widget, everything
        is created and in place
        
        This can be implemented in a subclass."""

    def worker_changed(self):
        pass

class Wizard(gobject.GObject):
    __gsignals__ = {
        'finished' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                      ())
        }
    
    def __init__(self):
        self.__gobject_init__()
        self.wtree = gtk.glade.XML(os.path.join(configure.gladedir, 'wizard.glade'))
        for widget in self.wtree.get_widget_prefix(''):
            setattr(self, widget.get_name(), widget)
        self.wtree.signal_autoconnect(self)
        self._use_main = True
        self.steps = []
        self.stack = Stack()
        self.current_step = None
        self._last_worker = 0 # combo id last worker from step to step
        self._worker_box = None # gtk.Widget containing worker combobox

    def debug(self, string):
        pass

    def __getitem__(self, stepname):
        for item in self.steps:
            if item.get_name() == stepname:
                return item
        else:
            raise KeyError

    def __len__(self):
        return len(self.steps)
            
    def error_dialog(self, message):
        """
        Show an error message dialog.
                                                                                
        @param message the message to display.
        @param parent the gtk.Window parent window.
        @param response whether the error dialog should go away after response.
                                                                                         returns: the error dialog.
        """
        d = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL,
                              gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
                              message)
        d.connect("response", lambda self, response: self.destroy())
        d.show_all()
        return d

    def get_step_option(self, stepname, option):
        state = self.get_step_options(stepname)
        return state[option]

    def get_step_options(self, stepname):
        step = self[stepname]
        return self.get_step_state(step)
    
    def get_step_state(self, step):
        state = step.get_state()
        dict = {}
        for key, widget in state.items():
            dict[key] = widget.get_state()
        return dict
    
    def block_next(self, block):
        self.button_next.set_sensitive(not block)

    def block_prev(self, block):
        self.button_prev.set_sensitive(not block)

    def add_step(self, step_class, initial=False):
        # If we don't have step_name set, count it as a base class
        name = step_class.step_name
        if name == None:
            return
        
        #if self.steps.has_key(name):
        #    raise TypeError("%s added twice" % name)

        step = step_class(self)
        self.steps.append(step)

        if step.__dict__.has_key('get_state'):
            state = self.get_step_state(step)
            assert type(state) == dict
            assert state, state
        
        step.setup()

        if initial:
            self.stack.push(step)

    def set_step(self, step):
        # Remove previous step
        map(self.content_area.remove, self.content_area.get_children())

        # Add current
        widget = step.get_main_widget()
        self.content_area.add(widget)
#        self.content_area.set_child_packing(widget, False, True, 0, 0)

        self._append_workers(step)
#FIX        icon_filename = os.path.join(configure.imagedir, 'wizard', step.icon)
#FIX        self.image_icon.set_from_file(icon_filename)
            
        self.label_title.set_markup('<span size="x-large">' + escape(step.get_name()) + '</span>')

        if self.current_step:
            self.current_step.deactivated()

        self.current_step = step
        
        self.update_sidebar(step)
        self.update_buttons(has_next=True)

        self._setup_worker(step)
        step.before_show()

        self.debug('showing step %r' % step)
        widget.show()
        step.activated()

    def _combobox_worker_changed(self, combobox, step):
        self._last_worker = combobox.get_active()
        self._setup_worker(step)
        step.worker_changed()
        
    def _append_workers(self, step):
        # called for each new page to put in the worker drop down box
        # if the step needs a worker
        self.combobox_worker = None
        return
        # Horizontal, under step
        hbox = gtk.HBox()
        self.content_area.pack_end(hbox, False, False)
        hbox.set_border_width(6)
        hbox.show()
        
        frame = gtk.Frame('Worker')
        hbox.pack_end(frame, False, False, 0)
        frame.show()

        # Internal, so we can add border width
        box = gtk.HBox()
        frame.add(box)
        box.set_border_width(6)
        box.show()
        self.combobox_worker = gtk.combo_box_new_text()
        box.pack_start(self.combobox_worker, False, False, 6)
        self.combobox_worker.connect('changed',
                                     self._combobox_worker_changed, step)
        self.combobox_worker.show()

    def show_previous(self):
        step = self.stack.pop()

        self._setup_worker(step)

        prev_step = self.stack.peek()
        
        self.set_step(prev_step)
        self._set_worker_from_step(prev_step)

        self.update_buttons(has_next=True)

    def _setup_worker(self, step):
        # get name of active worker
        if self.combobox_worker:
            model = self.combobox_worker.get_model()
            iter = self.combobox_worker.get_active_iter()
            if iter:
                text = model.get(iter, 0)[0]
                self.debug('%r setting worker to %s' % (step, text))
                step.worker = text
                return

        self.debug('%r no worker set' % step)
            
    def _set_worker_from_step(self, step):
        if not hasattr(step, 'worker'):
            return

        model = self.combobox_worker.get_model()
        current_text = step.worker
        for row in model:
            text = model.get(row.iter, 0)[0]
            if current_text == text:
                self.combobox_worker.set_active_iter(row.iter)
                break

    def show_next(self):
        next = self.current_step.get_next()
        if not next:
            self.finish(save=True)
            return

        self._setup_worker(self.current_step)

        try:
            next_step = self[next]
        except KeyError:
            raise TypeError("Wizard step %s is missing" % `next`)

        
        next_step.visited = True

        self.stack.push(next_step)
        self.set_step(next_step)

        has_next = not hasattr(next_step, 'last_step')
        self.update_buttons(has_next)

    def update_buttons(self, has_next):
        # update the forward and next buttons
        # has_next: whether or not there is a next step
        if len(self.stack) == 1:
            self.button_prev.set_sensitive(False)
        else:
            self.button_prev.set_sensitive(True)

        # XXX: Use the current step, not the one on the top of the stack
        if has_next:
            self.button_next.set_label(gtk.STOCK_GO_FORWARD)
        else:
            # use APPLY, just like in gnomemeeting
            self.button_next.set_label(gtk.STOCK_APPLY)

    def _sidebar_clean(self):
        # First remove the old the VBox if we can find one
        parent = self.vbox_sidebar.get_parent()
        if parent:
            parent.remove(self.vbox_sidebar)
        else:
            parent = self.eventbox_sidebar

        self.vbox_sidebar = gtk.VBox()
        self.vbox_sidebar.set_border_width(5)
        self.vbox_sidebar.set_size_request(200, -1)
        parent.add(self.vbox_sidebar)

    def _sidebar_add_placeholder(self):
        # Placeholder label, which expands vertically
        ph = gtk.Label()
        ph.show()
        self.vbox_sidebar.pack_start(ph)
        
    def _sidebar_add_step(self, step, name, active, padding):
        hbox = gtk.HBox(0, False)
        hbox.show()

        text = escape(name)
        button = gtk.Button('')

        label = button.get_children()[0]
        label.set_padding(padding, 0)
        label.set_alignment(0, 0.5)
        button.set_relief(gtk.RELIEF_NONE)
        hbox.pack_start(button, True, True)
        self.vbox_sidebar.pack_start(hbox, False, False)

        if not step:
            steps = [step for step in self.steps
                              if getattr(step, 'section_name', '') == name]
            assert len(steps) == 1
            step = steps[0]

        def button_clicked_cb(button, step):
            self.set_step(step)
            
        if step:
            button.connect('clicked', button_clicked_cb, step)
        else:
            button.connect('clicked', button_clicked_cb, steps[0])
            
        current = self.current_step
        if current == step:
            button.set_sensitive(False)
            
        if not active and not step.visited:
            markup = '%s' % name
            button.set_sensitive(False)
        else:
            markup = '%s' % (name)
            button.set_property('can_focus', False)
            
        label.set_markup(markup)

        button.show()
        return button
    
    def _sidebar_add_substeps(self, section):
        filtered_steps = [step for step in self.steps
                                   if (step.get_section() == section and
                                       step.visited == True and
                                       not hasattr(step, 'section_name'))]
        for step in filtered_steps:
            label = step.get_sidebar_name()
            self._sidebar_add_step(step, label, True, 20)

    def update_sidebar(self, step):
        current = step.get_section()

        self._sidebar_clean()
        
        sidebar_steps = ('Welcome',
                         'Login')
        active = True
        for stepname in sidebar_steps:
            self._sidebar_add_step(None, stepname, active, 10)

            if current == stepname:
                self._sidebar_add_substeps(stepname)
                self._sidebar_add_placeholder()
                active = False
            else:
                continue
            
        self.vbox_sidebar.show()
        
    def on_wizard_delete_event(self, wizard, event):
        self.finish(self._use_main, save=False)

    def on_button_prev_clicked(self, button):
        self.show_previous()

    def on_button_next_clicked(self, button):
        self.show_next()

    def finish(self, main=True, save=True):
        if save:
            self.emit('finished')
        
        if self._use_main:
            try:
                gtk.main_quit()
            except RuntimeError:
                pass

    def hide(self):
        self.window.hide()

    def run(self, interactive, main=True):
        self._use_main = main
        
        if not self.stack:
            raise TypeError("need an initial step")

        self.set_step(self.stack.peek())
        
        if not interactive:
            while self.current_step.get_next():
                self.show_next()
                
            return self.finish(False)

        self.window.show()
        if not self._use_main:
            return
        
        try:
            gtk.main()
        except KeyboardInterrupt:
            pass


    def load_steps(self):
        global _steps
        import steps
        
        self.add_step(_steps[0], initial=True)
        
        for step_class in _steps[1:]:
            self.add_step(step_class)
            

        return dict
#gobject.type_register(Wizard)

_steps = []

        
def register_step(klass):
    global _steps

    _steps.append(klass)

def get_steps():
    global _steps
    return _steps




# -*- Mode: Python; -*-
# vi:si:et:sw=4:sts=4:ts=4
#

# U8MG - an utf8 migration tool
#
# Copyright (C) 2005 Tollef Fog Heen
#
# Wizards taken from flumotion,
# Copyright (C) 2004,2005 Fluendo, S.L. (www.fluendo.com). All rights reserved.

# This file may be distributed and/or modified under the terms of
# the GNU General Public License version 2 as published by
# the Free Software Foundation.
# This file is distributed without any warranty; without even the implied
# warranty of merchantability or fitness for a particular purpose.
# See "LICENSE.GPL" in the source distribution for more information.




import ConfigParser

class gdmConfigParser(ConfigParser.RawConfigParser):

    def write(self, fp):
        """Write an .ini-format representation of the configuration state."""
        if self._defaults:
            fp.write("[%s]\n" % DEFAULTSECT)
            for (key, value) in self._defaults.items():
                fp.write("%s=%s\n" % (key, str(value).replace('\n', '\n\t')))
            fp.write("\n")
        for section in self._sections:
            fp.write("[%s]\n" % section)
            for (key, value) in self._sections[section].items():
                if key != "__name__":
                    fp.write("%s=%s\n" %
                             (key, str(value).replace('\n', '\n\t')))
            fp.write("\n")

    def optionxform(self, optionstr):
        return optionstr.capitalize()



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

# U8MG - an utf8 migration tool
#
# Copyright (C) 2005 Tollef Fog Heen
#
# Wizards taken from flumotion,
# Copyright (C) 2004,2005 Fluendo, S.L. (www.fluendo.com). All rights reserved.

# This file may be distributed and/or modified under the terms of
# the GNU General Public License version 2 as published by
# the Free Software Foundation.
# This file is distributed without any warranty; without even the implied
# warranty of merchantability or fitness for a particular purpose.
# See "LICENSE.GPL" in the source distribution for more information.

import gtk, os, gobject
        
import wizard

def findMatchingLocales(search, encoding = None):
    f = open("/usr/share/i18n/SUPPORTED")
    ret = []
    for line in f.readlines():
        (l, e) = line.strip().split(" ")
        if search in l and (e == encoding or encoding is None):
            ret.append(line.split(" ")[0].strip())
    return ret


class Welcome(wizard.WizardStep):
    step_name = 'Welcome'
    glade_file = 'wizard_welcome.glade'
    section = 'Welcome'
    section_name = 'Welcome'
    icon = 'wizard.png'
    has_worker = False
    def get_next(self):
        return 'Login' 
wizard.register_step(Welcome)


class Login(wizard.WizardStep):
    step_name = 'Login'
    glade_file = 'wizard_login.glade'
    section = 'Login'
    section_name = 'Login'
    icon = 'wizard.png'
    has_worker = False
    currentLocale = ""
    currentEncoding = ""
    newLocale = ""
    newEncoding = ""

    def getNewLocale(self):
        if self.currentLocale == 'C':
            return self.cbNewLocale.get_model()[self.cbNewLocale.get_active()][0]
        else:
            return self.lblNewLocale.get_text()

    def get_next(self):
        return "FileNameConversion"

    def before_show(self):
        if self.currentEncoding == 'UTF-8':
            self.lblLocaleInfo.set_label("Your login configuration is already UTF-8 enabled")
            self.tblLocaleInfo.hide()
        self.lblCurrentLocale.set_label(self.currentLocale)
        self.lblCurrentEncoding.set_label(self.currentEncoding)
        self.lblNewLocale.set_label(self.newLocale)
        self.lblNewEncoding.set_label(self.newEncoding)
        if self.currentLocale == 'C':
            self.lblNewLocale.hide()
            self.cbNewLocale.set_model(gtk.ListStore(gobject.TYPE_STRING))
            for t in findMatchingLocales("en_", encoding = "UTF-8"):
                print t
                self.cbNewLocale.get_model().append(row=[t])

            cell = gtk.CellRendererText()
            self.cbNewLocale.pack_start(cell, True)
            self.cbNewLocale.add_attribute(cell, 'text', 0)
            self.cbNewLocale.set_active(0)
            self.cbNewLocale.show()
        else:
            self.lblNewLocale.show()
            self.cbNewLocale.hide()
            
wizard.register_step(Login)


class FileNameConversion(wizard.WizardStep):
    step_name = 'FileNameConversion'
    glade_file = 'wizard_convertfiles.glade'
    section = 'Convert file names'
    section_name = 'Convert file names'
    icon = 'wizard.png'
    last_step = True
    has_worker = False
    model = gtk.ListStore(str, str, str)
    column = gtk.TreeViewColumn("File name")
    currentEncoding = ""

    def __init__(self, *params):
        super(FileNameConversion, self).__init__(params)
        self.renderer = gtk.CellRendererText()
        self.column.pack_start(self.renderer, True)
        self.column.set_attributes(self.renderer, text=2)
        self.renderer.set_property('editable', True)

    def edited_cb(self, cell, path, new_text, user_data):
        self.model[path][1] = new_text
        return

    def before_show(self):

        filenames = []
        for root, dirs, files in os.walk(os.path.expanduser('~'), topdown=False):
            for f in files+dirs:
                try:
                    unicode(f, "utf-8")
                except UnicodeDecodeError:
                    try:
                        filenames.append((root+"/"+f,
                                          root+"/"+unicode(f, self.currentEncoding).encode('utf-8'),
                                          unicode(root+"/"+f, self.currentEncoding)))
                    except UnicodeDecodeError:
                        # Sane fallback
                        filenames.append((root+"/"+f,
                                          root+"/"+unicode(f, "iso8859-1").encode('utf-8'),
                                          unicode(root+"/"+f, "iso8859-1")))

        self.set_files(filenames)

        if len(filenames) == 0:
            self.lblConvertFilesMessage.set_text("All your files have UTF-8 compatible names.  No renames needed.")
            self.scrolledwindow1.hide()
        else:
            self.tvFileNames.set_model(self.model)
            self.tvFileNames.append_column(self.column)
            self.tvFileNames.show_all()

            self.scrolledwindow1.set_resize_mode(gtk.RESIZE_IMMEDIATE)
            self.scrolledwindow1.resize_children()
            self.renderer.connect("edited", self.edited_cb, self.model)
                
    def set_files(self, files):
        for f in files:
            self.model.append(f)

    def get_files(self):
        files = []
        treeiter = self.model.get_iter_first()
        while treeiter is not None:
            files.append(self.model.get(treeiter, 0, 1))
            treeiter = self.model.iter_next(treeiter)
        return files

    def get_next(self):
        return None

wizard.register_step(FileNameConversion)





# -*- Mode: Python; test-case-name: flumotion.test.test_wizard -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# Flumotion - a streaming media server
# Copyright (C) 2004,2005 Fluendo, S.L. (www.fluendo.com). All rights reserved.

# This file may be distributed and/or modified under the terms of
# the GNU General Public License version 2 as published by
# the Free Software Foundation.
# This file is distributed without any warranty; without even the implied
# warranty of merchantability or fitness for a particular purpose.
# See "LICENSE.GPL" in the source distribution for more information.

# Licensees having purchased or holding a valid Flumotion Advanced
# Streaming Server license may use this file in accordance with the
# Flumotion Advanced Streaming Server Commercial License Agreement.
# See "LICENSE.Flumotion" in the source distribution for more information.

# Headers in this file shall remain intact.

from flumotion.common import log
from flumotion.wizard import enums

class Component:
    def __init__(self, name, type, properties={}, worker=None):
        log.debug('Creating component %s (%s) worker=%r' % (
            name, type, worker))
        self.name = name
        self.type = type
        self.props = properties
        self.worker = worker
        self.eaters = []
        self.feeders = []
        
    def __repr__(self):
        return '<flumotion.wizard.save.Component name="%s">' % self.name

    def addEater(self, component):
        self.eaters.append(component)
        
    def addFeeder(self, component):
        self.feeders.append(component)

    def link(self, component):
        self.feeders.append(component)
        component.addEater(self)

    def getFeeders(self):
        s = []
        for source in self.feeders:
            if source.type == 'firewire':
                if self.name in ('video-encoder', 'video-overlay'):
                    feed = 'video'
                else:
                    feed = 'audio'
                s.append('%s:%s' % (source.name, feed))
            else:
                s.append(source.name)
                
        return s
    
    def toXML(self):
        if self.worker:
            extra = ' worker="%s"' % self.worker
        else:
            extra = ''
            
        s = '    <component name="%s" type="%s"%s>\n' % (self.name,
                                                         self.type,
                                                         extra)

        # XXX: Handle eaters properly
        if self.type == 'firewire':
            s += '      <feed>audio</feed>\n'
            s += '      <feed>video</feed>\n'
        else:
            s += '      <feed>default</feed>\n'

        for sourceName in self.getFeeders():
            s += "      <source>%s</source>\n" % sourceName
                    
                
        if self.props:
            s += "      <!-- properties -->\n"
            property_names = self.props.keys()
            property_names.sort()
            
            for name in property_names:
                value = self.props[name]
                s += "      <%s>%s</%s>\n" % (name, value, name)
            
        s += "    </component>\n"
        return s
    
    def printTree(self, indent=1):
        print indent * '*', self.name, self.type, \
              tuple([f.name for f in self.feeders]) or ''
        for eater in self.eaters:
            eater.printTree(indent+1)

class WizardSaver:
    def __init__(self, wizard):
        self.wizard = wizard

    def getVideoSource(self):
        options = self.wizard.get_step_options('Source')
        source = options['video']
        video_step = self.wizard[source.step]
        
        if hasattr(video_step, 'worker'):
            props = video_step.get_component_properties()
            worker = video_step.worker
        else:
            props = {}
            worker = self.wizard['Source'].worker

        return Component('video-source', source.component_type, props, worker)

    def getVideoOverlay(self, show_logo):
        # At this point we already know that we should overlay something
        step = self.wizard['Overlay']
        properties = step.get_component_properties()
        if show_logo:
            properties['fluendo_logo'] = True
            encoding_options = self.wizard.get_step_options('Encoding')
            if (encoding_options['format'] == enums.EncodingFormat.Ogg or
                encoding_options['video'] == enums.EncodingVideo.Theora):
                properties['xiph_logo'] = True

            license_options = self.wizard.get_step_options('Content License')
            if license_options['license'] == enums.LicenseType.CC:
                properties['cc_logo'] = True
            
        return Component('video-overlay', 'overlay', properties, step.worker)
        
    def getVideoEncoder(self):
        options = self.wizard.get_step_options('Encoding')
        encoder = options['video']
        encoder_step = self.wizard[encoder.step]
        return Component('video-encoder', encoder.component_type,
                         encoder_step.get_component_properties(),
                         encoder_step.worker)

    def getAudioSource(self, video_source):
        options = self.wizard.get_step_options('Source')
        source = options['audio']
        
        # If we selected firewire and have selected video
        # and the selected video is Firewire,
        #   return the source
        if (source == enums.AudioDevice.Firewire and video_source and
            options['video'] == enums.VideoDevice.Firewire):
            return video_source
        
        props = {}
        
        audio_step = self.wizard[source.step]
        
        if hasattr(audio_step, 'worker'):
            props = audio_step.get_component_properties()
            worker = audio_step.worker
        else:
            worker = self.wizard['Source'].worker
        
        return Component('audio-source', source.component_type, props, worker)

    def getAudioEncoder(self):
        options = self.wizard.get_step_options('Encoding')
        encoder = options['audio']
        
        if encoder == enums.EncodingAudio.Mulaw:
            props = {}
            # FIXME
            worker = None 
        else:
            encoder_step = self.wizard[encoder.step]
            props = encoder_step.get_component_properties()
            worker = encoder_step.worker
            
        return Component('audio-encoder', encoder.component_type, props, worker)

    def getMuxer(self, name):
        options = self.wizard.get_step_options('Encoding')
        step = self.wizard['Encoding']
        muxer = options['format']
        return Component('multiplexer-' + name, muxer.component_type,
                         worker=step.worker)

    def handleVideo(self, components):
        overlay_options = self.wizard.get_step_options('Overlay')
        has_overlay = overlay_options['show_logo'] or \
                      overlay_options['show_text']
        
        video_source =  self.getVideoSource()
        components.append(video_source)
            
        video_overlay = None
        video_encoder = self.getVideoEncoder()
            
        if has_overlay:
            video_overlay = self.getVideoOverlay(overlay_options['show_logo'])
            components.append(video_overlay)
                
        if video_overlay is not None:
            video_overlay.link(video_source)
            video_encoder.link(video_overlay)
        else:
            video_encoder.link(video_source)
        components.append(video_encoder)
        return video_encoder, video_source
            
    def handleAudio(self, components, video_source):
        audio_source = self.getAudioSource(video_source)
        # In case of firewire component, which can already be there
        if not audio_source in components:
            components.append(audio_source)
            
        audio_encoder = self.getAudioEncoder()
        components.append(audio_encoder)
        audio_encoder.link(audio_source)

        return audio_encoder
    
    def handleConsumers(self, components, audio_encoder, video_encoder):
        cons_options = self.wizard.get_step_options('Consumption')
        has_audio = self.wizard.get_step_option('Source', 'has_audio')
        has_video = self.wizard.get_step_option('Source', 'has_video')

        audio_muxer = None
        if has_audio:
            audio_muxer = self.getMuxer('audio')
            components.append(audio_muxer)
            audio_muxer.link(audio_encoder)
            
        video_muxer = None
        if has_video:
            video_muxer = self.getMuxer('video')
            components.append(video_muxer)
            video_muxer.link(video_encoder)

        steps = []
        both_muxer = None
        if has_audio and has_video:
            both_muxer = self.getMuxer('audio-video')
            components.append(both_muxer)
            both_muxer.link(video_encoder)
            both_muxer.link(audio_encoder)
            
            if cons_options['http']:
                if cons_options['http_audio_video']:
                    steps.append(('http_audio_video', 'http-audio-video',
                                  'http-streamer',
                                  'HTTP Streamer (audio & video)', both_muxer))
                if cons_options['http_audio']:
                    steps.append(('http_audio', 'http-audio', 'http-streamer',
                                  'HTTP Streamer (audio only)', audio_muxer))
                if cons_options['http_video']:
                    steps.append(('http_video', 'http-video', 'http-streamer',
                                  'HTTP Streamer (video only)', video_muxer))
            if cons_options['disk']:
                if cons_options['disk_audio_video']:
                    steps.append(('disk_audio_video', 'disk-audio-video',
                                  'disker', 'Disk (audio & video)',
                                  both_muxer))
                if cons_options['disk_audio']:
                    steps.append(('disk_audio', 'disk-audio', 'disker',
                                  'Disk (audio only)', audio_muxer))
                if cons_options['disk_video']:
                    steps.append(('disk_video', 'disk-video', 'disker',
                                  'Disk (video only)', video_muxer))
        elif has_video and not has_audio:
            if cons_options['http']:
                steps.append(('http_video', 'http-video', 'http-streamer',
                              'HTTP Streamer (video only)', video_muxer))
            if cons_options['disk']:
                steps.append(('disk_video', 'disk-video', 'disker',
                              'Disk (video only)', video_muxer))
        elif has_audio and not has_video:
            if cons_options['http']:
                steps.append(('http_audio', 'http-audio', 'http-streamer',
                              'HTTP Streamer (audio only)', audio_muxer))
            if cons_options['disk']:
                steps.append(('disk_audio', 'disk-audio', 'disker',
                              'Disk (audio only)', audio_muxer))
        else:
            raise AssertionError

        for key, name, type, step_name, muxer in steps:
            if not cons_options.has_key(key):
                continue
            step = self.wizard[step_name]
            consumer = Component(name, type, step.get_component_properties(),
                                 step.worker)
            consumer.link(muxer)
            components.append(consumer)

        # Removed unused ones
        if audio_muxer and not audio_muxer.eaters:
            components.remove(audio_muxer)
        if video_muxer and not video_muxer.eaters:
            components.remove(video_muxer)
        if both_muxer and not both_muxer.eaters:
            components.remove(both_muxer)

    def getComponents(self):
        source_options = self.wizard.get_step_options('Source')
        has_video = source_options['has_video']
        has_audio = source_options['has_audio']

        components = []
        
        video_encoder = None
        video_source = None
        if has_video:
            video_encoder, video_source = self.handleVideo(components)

        # Must do audio after video, in case of a firewire audio component
        # is selected together with a firewire video component
        audio_encoder = None
        if has_audio:
            audio_encoder = self.handleAudio(components, video_source)
            
        self.handleConsumers(components, audio_encoder, video_encoder)

        return components
    
    def getXML(self):
        components = self.getComponents()
        
        s = '<planet>\n'
        s += '  <flow>\n'
        for component in components:
            s += component.toXML()
        s += '  </flow>\n'
        s += '</planet>\n'

        return s

